@@ -84,7 +84,7 @@ class Cdx:
 
         endpoint = "https://web.archive.org/cdx/search/cdx"
         total_pages = _get_total_pages(self.url, self.user_agent)
-        #If we only have two or less pages of archives then we care for accuracy
+        # If we only have two or less pages of archives then we care for accuracy
         # pagination API can be lagged sometimes
         if use_page == True and total_pages >= 2:
             blank_pages = 0
@@ -194,8 +194,11 @@ class Cdx:
                 properties_len = len(properties)
                 if prop_values_len != properties_len:
                     raise WaybackError(
-                        "Snapshot returned by Cdx API has %s properties instead of expected %s properties.\nInvolved Snapshot : %s"
-                        % (prop_values_len, properties_len, snapshot)
+                        "Snapshot returned by Cdx API has {prop_values_len} properties instead of expected {properties_len} properties.\nInvolved Snapshot : {snapshot}".format(
+                            prop_values_len=prop_values_len,
+                            properties_len=properties_len,
+                            snapshot=snapshot,
+                        )
                     )
 
                 (
