@@ -1,6 +1,7 @@
 import os
 import re
 import sys
+import json
 import random
 import string
 import argparse
@@ -20,13 +21,14 @@ def _save(obj):
         if "No archive URL found in the API response" in e:
             return (
                 "\n[waybackpy] Can not save/archive your link.\n[waybackpy] This "
-                "could happen because either your waybackpy (%s) is likely out of "
+                "could happen because either your waybackpy ({version}) is likely out of "
                 "date or Wayback Machine is malfunctioning.\n[waybackpy] Visit "
                 "https://github.com/akamhy/waybackpy for the latest version of "
-                "waybackpy.\n[waybackpy] API response Header :\n%s"
-                % (__version__, header)
+                "waybackpy.\n[waybackpy] API response Header :\n{header}".format(
+                    version=__version__, header=header
+                )
             )
-        return WaybackError(err)
+        raise WaybackError(err)
 
 
 def _archive_url(obj):
@@ -34,7 +36,7 @@ def _archive_url(obj):
 
 
 def _json(obj):
-    return obj.JSON
+    return json.dumps(obj.JSON)
 
 
 def no_archive_handler(e, obj):
@@ -45,11 +47,13 @@ def no_archive_handler(e, obj):
         if "github.com/akamhy/waybackpy" in ua:
             ua = "YOUR_USER_AGENT_HERE"
         return (
-            "\n[Waybackpy] Can not find archive for '%s'.\n[Waybackpy] You can"
+            "\n[Waybackpy] Can not find archive for '{url}'.\n[Waybackpy] You can"
             " save the URL using the following command:\n[Waybackpy] waybackpy --"
-            'user_agent "%s" --url "%s" --save' % (url, ua, url)
+            'user_agent "{user_agent}" --url "{url}" --save'.format(
+                url=url, user_agent=ua
+            )
         )
-    return WaybackError(e)
+    raise WaybackError(e)
 
 
 def _oldest(obj):
@@ -85,23 +89,40 @@ def _near(obj, args):
         return no_archive_handler(e, obj)
 
 
-def _save_urls_on_file(input_list, live_url_count):
-    m = re.search("https?://([A-Za-z_0-9.-]+).*", input_list[0])
-
-    domain = "domain-unknown"
-    if m:
-        domain = m.group(1)
-
+def _save_urls_on_file(url_gen):
+    domain = None
+    sys_random = random.SystemRandom()
     uid = "".join(
-        random.choice(string.ascii_lowercase + string.digits) for _ in range(6)
+        sys_random.choice(string.ascii_lowercase + string.digits) for _ in range(6)
     )
+    url_count = 0
 
-    file_name = "%s-%d-urls-%s.txt" % (domain, live_url_count, uid)
-    file_content = "\n".join(input_list)
-    file_path = os.path.join(os.getcwd(), file_name)
-    with open(file_path, "w+") as f:
-        f.write(file_content)
-    return "%s\n\n'%s' saved in current working directory" % (file_content, file_name)
+    for url in url_gen:
+        url_count += 1
+        if not domain:
+            m = re.search("https?://([A-Za-z_0-9.-]+).*", url)
+
+            domain = "domain-unknown"
+
+            if m:
+                domain = m.group(1)
+
+            file_name = "{domain}-urls-{uid}.txt".format(domain=domain, uid=uid)
+            file_path = os.path.join(os.getcwd(), file_name)
+            if not os.path.isfile(file_path):
+                open(file_path, "w+").close()
+
+        with open(file_path, "a") as f:
+            f.write("{url}\n".format(url=url))
+
+        print(url)
+
+    if url_count > 0:
+        return "\n\n'{file_name}' saved in current working directory".format(
+            file_name=file_name
+        )
+    else:
+        return "No known URLs found. Please try a diffrent input!"
 
 
 def _known_urls(obj, args):
@@ -109,21 +130,16 @@ def _known_urls(obj, args):
     Known urls for a domain.
     """
 
-    subdomain = False
-    if args.subdomain:
-        subdomain = True
+    subdomain = True if args.subdomain else False
 
-    alive = False
-    if args.alive:
-        alive = True
+    url_gen = obj.known_urls(subdomain=subdomain)
 
-    url_list = obj.known_urls(alive=alive, subdomain=subdomain)
-    total_urls = len(url_list)
-
-    if total_urls > 0:
-        return _save_urls_on_file(url_list, total_urls)
-
-    return "No known URLs found. Please try a diffrent domain!"
+    if args.file:
+        return _save_urls_on_file(url_gen)
+    else:
+        for url in url_gen:
+            print(url)
+        return "\n"
 
 
 def _get(obj, args):
@@ -147,12 +163,11 @@ def _get(obj, args):
 
 def args_handler(args):
     if args.version:
-        return "waybackpy version %s" % __version__
+        return "waybackpy version {version}".format(version=__version__)
 
     if not args.url:
-        return (
-            "waybackpy %s \nSee 'waybackpy --help' for help using this tool."
-            % __version__
+        return "waybackpy {version} \nSee 'waybackpy --help' for help using this tool.".format(
+            version=__version__
         )
 
     obj = Url(args.url)
@@ -261,8 +276,12 @@ def add_knownUrlArg(knownUrlArg):
     )
     help_text = "Use with '--known_urls' to include known URLs for subdomains."
     knownUrlArg.add_argument("--subdomain", "-sub", action="store_true", help=help_text)
-    help_text = "Only include live URLs. Will not inlclude dead links."
-    knownUrlArg.add_argument("--alive", "-a", action="store_true", help=help_text)
+    knownUrlArg.add_argument(
+        "--file",
+        "-f",
+        action="store_true",
+        help="Save the URLs in file at current directory.",
+    )
 
 
 def add_nearArg(nearArg):
