@@ -1,4 +1,5 @@
 import re
+import time
 import requests
 from .exceptions import WaybackError, URLError
 from datetime import datetime
@@ -11,32 +12,38 @@ quote = requests.utils.quote
 default_user_agent = "waybackpy python package - https://github.com/akamhy/waybackpy"
 
 
+def _latest_version(package_name, headers):
+    endpoint = "https://pypi.org/pypi/" + package_name + "/json"
+    json = _get_response(endpoint, headers=headers).json()
+    return json["info"]["version"]
+
+
 def _unix_ts_to_wayback_ts(unix_ts):
     return datetime.utcfromtimestamp(int(unix_ts)).strftime("%Y%m%d%H%M%S")
 
 
-def _add_payload(self, payload):
-    if self.start_timestamp:
-        payload["from"] = self.start_timestamp
+def _add_payload(instance, payload):
+    if instance.start_timestamp:
+        payload["from"] = instance.start_timestamp
 
-    if self.end_timestamp:
-        payload["to"] = self.end_timestamp
+    if instance.end_timestamp:
+        payload["to"] = instance.end_timestamp
 
-    if self.gzip != True:
+    if instance.gzip != True:
         payload["gzip"] = "false"
 
-    if self.match_type:
-        payload["matchType"] = self.match_type
+    if instance.match_type:
+        payload["matchType"] = instance.match_type
 
-    if self.filters and len(self.filters) > 0:
-        for i, f in enumerate(self.filters):
+    if instance.filters and len(instance.filters) > 0:
+        for i, f in enumerate(instance.filters):
             payload["filter" + str(i)] = f
 
-    if self.collapses and len(self.collapses) > 0:
-        for i, f in enumerate(self.collapses):
+    if instance.collapses and len(instance.collapses) > 0:
+        for i, f in enumerate(instance.collapses):
             payload["collapse" + str(i)] = f
 
-    payload["url"] = self.url
+    payload["url"] = instance.url
 
 
 def _ts(timestamp, data):
@@ -71,10 +78,10 @@ def _check_match_type(match_type, url):
     legal_match_type = ["exact", "prefix", "host", "domain"]
 
     if match_type not in legal_match_type:
-        raise WaybackError(
-            "%s is not an allowed match type.\nUse one from 'exact', 'prefix', 'host' or 'domain'"
-            % match_type
+        exc_message = "{match_type} is not an allowed match type.\nUse one from 'exact', 'prefix', 'host' or 'domain'".format(
+            match_type=match_type
         )
+        raise WaybackError(exc_message)
 
 
 def _check_collapses(collapses):
@@ -85,11 +92,11 @@ def _check_collapses(collapses):
     if len(collapses) == 0:
         return
 
-    for c in collapses:
+    for collapse in collapses:
         try:
             match = re.search(
                 r"(urlkey|timestamp|original|mimetype|statuscode|digest|length)(:?[0-9]{1,99})?",
-                c,
+                collapse,
             )
             field = match.group(1)
 
@@ -98,15 +105,17 @@ def _check_collapses(collapses):
                 N = match.group(2)
 
             if N:
-                if not (field + N == c):
+                if not (field + N == collapse):
                     raise Exception
             else:
-                if not (field == c):
+                if not (field == collapse):
                     raise Exception
 
         except Exception:
-            e = "collapse argument '%s' is not following the cdx collapse syntax." % c
-            raise WaybackError(e)
+            exc_message = "collapse argument '{collapse}' is not following the cdx collapse syntax.".format(
+                collapse=collapse
+            )
+            raise WaybackError(exc_message)
 
 
 def _check_filters(filters):
@@ -114,19 +123,23 @@ def _check_filters(filters):
         raise WaybackError("filters must be a list.")
 
     # [!]field:regex
-    for f in filters:
+    for _filter in filters:
         try:
             match = re.search(
                 r"(\!?(?:urlkey|timestamp|original|mimetype|statuscode|digest|length)):(.*)",
-                f,
+                _filter,
             )
 
             key = match.group(1)
             val = match.group(2)
 
         except Exception:
-            e = "Filter '%s' not following the cdx filter syntax." % f
-            raise WaybackError(e)
+            exc_message = (
+                "Filter '{_filter}' not following the cdx filter syntax.".format(
+                    _filter=_filter
+                )
+            )
+            raise WaybackError(exc_message)
 
 
 def _cleaned_url(url):
@@ -143,7 +156,8 @@ def _url_check(url):
     """
 
     if "." not in url:
-        raise URLError("'%s' is not a vaild URL." % url)
+        exc_message = "'{url}' is not a vaild URL.".format(url=url)
+        raise URLError(exc_message)
 
 
 def _full_url(endpoint, params):
@@ -154,7 +168,9 @@ def _full_url(endpoint, params):
             key = "filter" if key.startswith("filter") else key
             key = "collapse" if key.startswith("collapse") else key
             amp = "" if full_url.endswith("?") else "&"
-            full_url = full_url + amp + "%s=%s" % (key, quote(str(val)))
+            full_url = (
+                full_url + amp + "{key}={val}".format(key=key, val=quote(str(val)))
+            )
     return full_url
 
 
@@ -166,13 +182,15 @@ def _get_total_pages(url, user_agent):
     This func returns number of pages of archives (type int).
     """
     total_pages_url = (
-        "https://web.archive.org/cdx/search/cdx?url=%s&showNumPages=true" % url
+        "https://web.archive.org/cdx/search/cdx?url={url}&showNumPages=true".format(
+            url=url
+        )
     )
     headers = {"User-Agent": user_agent}
     return int((_get_response(total_pages_url, headers=headers).text).strip())
 
 
-def _archive_url_parser(header, url):
+def _archive_url_parser(header, url, latest_version=__version__, instance=None):
     """
     The wayback machine's save API doesn't
     return JSON response, we are required
@@ -194,10 +212,40 @@ def _archive_url_parser(header, url):
 
     If we found the archive URL we return it.
 
+    Return format:
+
+    web.archive.org/web/<TIMESTAMP>/<URL>
+
     And if we couldn't find it, we raise
     WaybackError with an error message.
     """
 
+    if "save redirected" in header and instance:
+        time.sleep(60)  # makeup for archive time
+
+        now = datetime.utcnow().timetuple()
+        timestamp = _wayback_timestamp(
+            year=now.tm_year,
+            month=now.tm_mon,
+            day=now.tm_mday,
+            hour=now.tm_hour,
+            minute=now.tm_min,
+        )
+
+        return_str = "web.archive.org/web/{timestamp}/{url}".format(
+            timestamp=timestamp, url=url
+        )
+        url = "https://" + return_str
+
+        headers = {"User-Agent": instance.user_agent}
+
+        res = _get_response(url, headers=headers)
+
+        if res.status_code < 400:
+            return "web.archive.org/web/{timestamp}/{url}".format(
+                timestamp=timestamp, url=url
+            )
+
     # Regex1
     m = re.search(r"Content-Location: (/web/[0-9]{14}/.*)", str(header))
     if m:
@@ -215,13 +263,44 @@ def _archive_url_parser(header, url):
     if m:
         return m.group(1)
 
-    raise WaybackError(
-        "No archive URL found in the API response. "
-        "If '%s' can be accessed via your web browser then either "
-        "this version of waybackpy (%s) is out of date or WayBack Machine is malfunctioning. Visit "
-        "'https://github.com/akamhy/waybackpy' for the latest version "
-        "of waybackpy.\nHeader:\n%s" % (url, __version__, str(header))
-    )
+    if instance:
+        newest_archive = None
+        try:
+            newest_archive = instance.newest()
+        except WaybackError:
+            pass  # We don't care as this is a save request
+
+        if newest_archive:
+            minutes_old = (
+                datetime.utcnow() - newest_archive.timestamp
+            ).total_seconds() / 60.0
+
+            if minutes_old <= 30:
+                archive_url = newest_archive.archive_url
+                m = re.search(r"web\.archive\.org/web/[0-9]{14}/.*", archive_url)
+                if m:
+                    instance.cached_save = True
+                    return m.group(0)
+
+    if __version__ == latest_version:
+        exc_message = (
+            "No archive URL found in the API response. "
+            "If '{url}' can be accessed via your web browser then either "
+            "Wayback Machine is malfunctioning or it refused to archive your URL."
+            "\nHeader:\n{header}".format(url=url, header=header)
+        )
+    else:
+        exc_message = (
+            "No archive URL found in the API response. "
+            "If '{url}' can be accessed via your web browser then either "
+            "this version of waybackpy ({version}) is out of date or WayBack "
+            "Machine is malfunctioning. Visit 'https://github.com/akamhy/waybackpy' "
+            "for the latest version of waybackpy.\nHeader:\n{header}".format(
+                url=url, version=__version__, header=header
+            )
+        )
+
+    raise WaybackError(exc_message)
 
 
 def _wayback_timestamp(**kwargs):
@@ -252,7 +331,13 @@ def _wayback_timestamp(**kwargs):
 
 
 def _get_response(
-    endpoint, params=None, headers=None, retries=5, return_full_url=False
+    endpoint,
+    params=None,
+    headers=None,
+    return_full_url=False,
+    retries=5,
+    backoff_factor=0.5,
+    no_raise_on_redirects=False,
 ):
     """
     This function is used make get request.
@@ -274,17 +359,31 @@ def _get_response(
 
     # From https://stackoverflow.com/a/35504626
     # By https://stackoverflow.com/users/401467/datashaman
+
     s = requests.Session()
+
     retries = Retry(
-        total=retries, backoff_factor=0.5, status_forcelist=[500, 502, 503, 504]
+        total=retries,
+        backoff_factor=backoff_factor,
+        status_forcelist=[500, 502, 503, 504],
     )
+
     s.mount("https://", HTTPAdapter(max_retries=retries))
+
     url = _full_url(endpoint, params)
+
     try:
         if not return_full_url:
             return s.get(url, headers=headers)
         return (url, s.get(url, headers=headers))
     except Exception as e:
-        exc = WaybackError("Error while retrieving %s" % url)
+        reason = str(e)
+        if no_raise_on_redirects:
+            if "Exceeded 30 redirects" in reason:
+                return
+        exc_message = "Error while retrieving {url}.\n{reason}".format(
+            url=url, reason=reason
+        )
+        exc = WaybackError(exc_message)
         exc.__cause__ = e
         raise exc
